@@ -28,7 +28,7 @@ BEGIN {
     }
 }
 
-our $VERSION = '0.32';
+our $VERSION = '0.33';
 
 sub _ANYPROCESS                    { return -1 }
 sub _COMMAND                       { return 0 }
@@ -37,6 +37,7 @@ sub _WIN32_ERROR_SHARING_VIOLATION { return 0x20 }
 sub _NUMBER_OF_MCOOKIE_BYTES       { return 16 }
 sub _MAX_DISPLAY_LENGTH            { return 10 }
 sub _NUMBER_OF_TERM_ATTEMPTS       { return 4 }
+sub _OLD_BROWSER_MAJOR_VERSION     { return 56 }
 
 sub new {
     my ( $class, %parameters ) = @_;
@@ -120,9 +121,9 @@ sub _launch {
         && ( $OSNAME ne 'cygwin' )
         && ( $self->_visible() )
         && ( !$ENV{DISPLAY} )
-        && ( $self->_xvfb_exists() ) )
+        && ( $self->_xvfb_exists() )
+        && ( $self->_launch_xvfb() ) )
     { # if not MacOS or Win32 and no DISPLAY variable, launch Xvfb if at all possible
-        $self->_launch_xvfb();
         local $ENV{DISPLAY}    = $self->_xvfb_display();
         local $ENV{XAUTHORITY} = $self->_xvfb_xauthority();
         return $self->_launch_unix(@arguments);
@@ -369,6 +370,11 @@ sub _launch_xvfb {
             if ( $display_number !~ /^\d+$/smx ) {
                 sleep 1;
             }
+            waitpid $pid, POSIX::WNOHANG();
+            if ( !kill 0, $pid ) {
+                Carp::carp('Unable to start Xvfb');
+                return 0;
+            }
         }
         $self->{_xvfb_display_number} = $display_number;
         close $display_no_handle
@@ -707,14 +713,19 @@ sub new_session {
     }
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:NewSession', $parameters ] );
+        [
+            _COMMAND(),                              $message_id,
+            $self->_command('WebDriver:NewSession'), $parameters
+        ]
+    );
     my $response;
     eval { $response = $self->_get_response($message_id); } or do {
         if (
             ( ref $EVAL_ERROR )
             && ( ( ref $EVAL_ERROR ) eq
                 'Firefox::Marionette::Exception::Response' )
-            && ( $EVAL_ERROR->error() eq 'invalid session id' )
+            && (   ( $EVAL_ERROR->error() eq 'invalid session id' )
+                || ( $EVAL_ERROR->error() eq 'unknown command' ) )
           )
         {
             my $fallback_message_id = $self->_new_message_id();
@@ -728,8 +739,15 @@ sub new_session {
         }
     };
     $self->{session_id} = $response->result()->{sessionId};
-    return ( $self->{session_id},
-        $self->_create_capabilities( $response->result()->{capabilities} ) );
+    my $new =
+      $self->_create_capabilities( $response->result()->{capabilities} );
+    $self->{_browser_version} = $new->browser_version();
+    return ( $self->{session_id}, $new );
+}
+
+sub browser_version {
+    my ($self) = @_;
+    return $self->{_browser_version};
 }
 
 sub _create_capabilities {
@@ -738,6 +756,12 @@ sub _create_capabilities {
     if ( ($pid) && ( $OSNAME eq 'cygwin' ) ) {
         $pid = $self->_pid();
     }
+    my $headless = $self->_visible() ? 0 : 1;
+    if ( defined $parameters->{'moz:headless'} ) {
+        if ( $parameters->{'moz:headless'} != $headless ) {
+            Carp::croak('moz:headless has not been determined correctly');
+        }
+    }
     return Firefox::Marionette::Capabilities->new(
         accept_insecure_certs => $parameters->{acceptInsecureCerts} ? 1 : 0,
         page_load_strategy    => $parameters->{pageLoadStrategy},
@@ -746,15 +770,15 @@ sub _create_capabilities {
             script    => $parameters->{timeouts}->{script},
             implicit  => $parameters->{timeouts}->{implicit},
         ),
-        browser_version          => $parameters->{browserVersion},
-        platform_name            => $parameters->{platformName},
-        rotatable                => $parameters->{rotatable} ? 1 : 0,
+        browser_version => $parameters->{browserVersion},
+        platform_name   => $parameters->{platformName},
+        rotatable => $parameters->{rotatable} ? 1 : 0,
         platform_version         => $parameters->{platformVersion},
         moz_profile              => $parameters->{'moz:profile'},
         moz_webdriver_click      => $parameters->{'moz:webdriverClick'} ? 1 : 0,
         moz_process_id           => $pid,
         browser_name             => $parameters->{browserName},
-        moz_headless             => $parameters->{'moz:headless'} ? 1 : 0,
+        moz_headless             => $headless,
         moz_accessibility_checks => $parameters->{'moz:accessibilityChecks'}
         ? 1
         : 0,
@@ -775,8 +799,10 @@ sub list {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'WebDriver:FindElements', { using => $using, value => $value }
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:FindElements'),
+            { using => $using, value => $value }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -800,7 +826,7 @@ sub add_cookie {
         [
             _COMMAND(),
             $message_id,
-            'WebDriver:AddCookie',
+            $self->_command('WebDriver:AddCookie'),
             {
                 cookie => {
                     httpOnly => $cookie->http_only()
@@ -837,7 +863,8 @@ sub is_selected {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:IsElementSelected', { id => $element->uuid() }
+            $self->_command('WebDriver:IsElementSelected'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -861,7 +888,8 @@ sub is_enabled {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:IsElementEnabled', { id => $element->uuid() }
+            $self->_command('WebDriver:IsElementEnabled'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -885,7 +913,8 @@ sub is_displayed {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:IsElementDisplayed', { id => $element->uuid() }
+            $self->_command('WebDriver:IsElementDisplayed'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -914,7 +943,9 @@ sub type {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id, 'WebDriver:ElementSendKeys',
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:ElementSendKeys'),
             { id => $element->uuid(), text => $text }
         ]
     );
@@ -926,7 +957,8 @@ sub delete_session {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:DeleteSession' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:DeleteSession') ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -935,7 +967,10 @@ sub minimise {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:MinimizeWindow' ] );
+        [
+            _COMMAND(), $message_id, $self->_command('WebDriver:MinimizeWindow')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -944,7 +979,10 @@ sub maximise {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:MaximizeWindow' ] );
+        [
+            _COMMAND(), $message_id, $self->_command('WebDriver:MaximizeWindow')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -952,16 +990,106 @@ sub maximise {
 sub refresh {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
-    $self->_send_request( [ _COMMAND(), $message_id, 'WebDriver:Refresh' ] );
+    $self->_send_request(
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:Refresh') ] );
     my $response = $self->_get_response($message_id);
     return $self;
 }
 
+my %_deprecated_commands = (
+    'WebDriver:GetCapabilities'              => 'getSessionCapabilities',
+    'Marionette:Quit'                        => 'quit',
+    'Marionette:SetContext'                  => 'setContext',
+    'Marionette:GetContext'                  => 'getContext',
+    'Marionette:AcceptConnections'           => 'acceptConnections',
+    'Addon:Install'                          => 'addon:install',
+    'Addon:Uninstall'                        => 'addon:uninstall',
+    'WebDriver:AcceptDialog'                 => 'acceptDialog',
+    'WebDriver:AddCookie'                    => 'addCookie',
+    'WebDriver:Back'                         => 'goBack',
+    'WebDriver:CloseChromeWindow'            => 'closeChromeWindow',
+    'WebDriver:CloseWindow'                  => 'close',
+    'WebDriver:DeleteAllCookies'             => 'deleteAllCookies',
+    'WebDriver:DeleteCookie'                 => 'deleteCookie',
+    'WebDriver:DeleteSession'                => 'deleteSession',
+    'WebDriver:DismissAlert'                 => 'dismissDialog',
+    'WebDriver:GetWindowType'                => 'getWindowType',
+    'WebDriver:DismissAlert'                 => 'dismissDialog',
+    'WebDriver:ElementClear'                 => 'clearElement',
+    'WebDriver:ElementClick'                 => 'clickElement',
+    'WebDriver:ElementSendKeys'              => 'sendKeysToElement',
+    'WebDriver:ExecuteAsyncScript'           => 'executeAsyncScript',
+    'WebDriver:ExecuteScript'                => 'executeScript',
+    'WebDriver:FindElement'                  => 'findElement',
+    'WebDriver:FindElements'                 => 'findElements',
+    'WebDriver:Forward'                      => 'goForward',
+    'WebDriver:FullscreenWindow'             => 'fullscreenWindow',
+    'WebDriver:GetActiveElement'             => 'getActiveElement',
+    'WebDriver:GetActiveFrame'               => 'getActiveFrame',
+    'WebDriver:GetAlertText'                 => 'getTextFromDialog',
+    'WebDriver:GetCapabilities'              => 'getSessionCapabilities',
+    'WebDriver:GetChromeWindowHandle'        => 'getChromeWindowHandle',
+    'WebDriver:GetChromeWindowHandles'       => 'getChromeWindowHandles',
+    'WebDriver:GetCookies'                   => 'getCookies',
+    'WebDriver:GetCurrentChromeWindowHandle' => 'getChromeWindowHandle',
+    'WebDriver:GetCurrentURL'                => 'getCurrentUrl',
+    'WebDriver:GetElementAttribute'          => 'getElementAttribute',
+    'WebDriver:GetElementCSSValue'           => 'getElementValueOfCssProperty',
+    'WebDriver:GetElementProperty'           => 'getElementProperty',
+    'WebDriver:GetElementRect'               => 'getElementRect',
+    'WebDriver:GetElementTagName'            => 'getElementTagName',
+    'WebDriver:GetElementText'               => 'getElementText',
+    'WebDriver:GetPageSource'                => 'getPageSource',
+    'WebDriver:GetScreenOrientation'         => 'getScreenOrientation',
+    'WebDriver:GetTimeouts'                  => 'getTimeouts',
+    'WebDriver:GetTitle'                     => 'getTitle',
+    'WebDriver:GetWindowHandle'              => 'getWindowHandle',
+    'WebDriver:GetWindowHandles'             => 'getWindowHandles',
+    'WebDriver:GetWindowRect'                => 'getWindowRect',
+    'WebDriver:IsElementDisplayed'           => 'isElementDisplayed',
+    'WebDriver:IsElementEnabled'             => 'isElementEnabled',
+    'WebDriver:IsElementSelected'            => 'isElementSelected',
+    'WebDriver:MinimizeWindow'               => 'minimizeWindow',
+    'WebDriver:MaximizeWindow'               => 'maximizeWindow',
+    'WebDriver:Navigate'                     => 'get',
+    'WebDriver:NewSession'                   => 'newSession',
+    'WebDriver:PerformActions'               => 'performActions',
+    'WebDriver:Refresh'                      => 'refresh',
+    'WebDriver:ReleaseActions'               => 'releaseActions',
+    'WebDriver:SendAlertText'                => 'sendKeysToDialog',
+    'WebDriver:SetScreenOrientation'         => 'setScreenOrientation',
+    'WebDriver:SetTimeouts'                  => 'setTimeouts',
+    'WebDriver:SetWindowRect'                => 'setWindowRect',
+    'WebDriver:SwitchToFrame'                => 'switchToFrame',
+    'WebDriver:SwitchToParentFrame'          => 'switchToParentFrame',
+    'WebDriver:SwitchToShadowRoot'           => 'switchToShadowRoot',
+    'WebDriver:SwitchToWindow'               => 'switchToWindow',
+    'WebDriver:TakeScreenshot'               => 'takeScreenshot',
+);
+
+sub _command {
+    my ( $self, $command ) = @_;
+    if ( defined $self->browser_version() ) {
+        my ( $major, $minor, $patch ) = split /[.]/smx,
+          $self->browser_version();
+        if ( $major < _OLD_BROWSER_MAJOR_VERSION() ) {
+            if ( $_deprecated_commands{$command} ) {
+                return $_deprecated_commands{$command};
+            }
+        }
+    }
+    return $command;
+}
+
 sub capabilities {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetCapabilities' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetCapabilities')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self->_create_capabilities( $response->result()->{capabilities} );
 }
@@ -970,7 +1098,11 @@ sub delete_cookies {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:DeleteAllCookies' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:DeleteAllCookies')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -980,7 +1112,8 @@ sub delete_cookie {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id, 'WebDriver:DeleteCookie', { name => $name }
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:DeleteCookie'), { name => $name }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -990,7 +1123,8 @@ sub delete_cookie {
 sub cookies {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
-    $self->_send_request( [ _COMMAND(), $message_id, 'WebDriver:GetCookies' ] );
+    $self->_send_request(
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetCookies') ] );
     my $response = $self->_get_response($message_id);
     return map {
         Firefox::Marionette::Cookie->new(
@@ -1022,7 +1156,8 @@ sub tag_name {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:GetElementTagName', { id => $element->uuid() }
+            $self->_command('WebDriver:GetElementTagName'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1033,7 +1168,8 @@ sub window_rect {
     my ( $self, $new ) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetWindowRect' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetWindowRect') ]
+    );
     my $response = $self->_get_response($message_id);
     my $old      = Firefox::Marionette::Window::Rect->new(
         pos_x  => $response->result()->{x},
@@ -1048,7 +1184,7 @@ sub window_rect {
             [
                 _COMMAND(),
                 $message_id,
-                'WebDriver:SetWindowRect',
+                $self->_command('WebDriver:SetWindowRect'),
                 {
                     x      => $new->pos_x(),
                     y      => $new->pos_y(),
@@ -1078,7 +1214,8 @@ sub rect {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:GetElementRect', { id => $element->uuid() }
+            $self->_command('WebDriver:GetElementRect'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1106,7 +1243,8 @@ sub text {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:GetElementText', { id => $element->uuid() }
+            $self->_command('WebDriver:GetElementText'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1129,7 +1267,8 @@ sub clear {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:ElementClear', { id => $element->uuid() }
+            $self->_command('WebDriver:ElementClear'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1152,7 +1291,8 @@ sub click {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:ElementClick', { id => $element->uuid() }
+            $self->_command('WebDriver:ElementClick'),
+            { id => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1163,7 +1303,7 @@ sub timeouts {
     my ( $self, $new ) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetTimeouts' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetTimeouts') ] );
     my $response = $self->_get_response($message_id);
     my $old      = Firefox::Marionette::Timeouts->new(
         page_load => $response->result()->{pageLoad},
@@ -1176,7 +1316,7 @@ sub timeouts {
             [
                 _COMMAND(),
                 $message_id,
-                'WebDriver:SetTimeouts',
+                $self->_command('WebDriver:SetTimeouts'),
                 {
                     pageLoad => $new->page_load(),
                     script   => $new->script(),
@@ -1193,7 +1333,11 @@ sub active_element {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetActiveElement' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetActiveElement')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return Firefox::Marionette::Element->new( $self,
         %{ $response->result()->{value} } );
@@ -1203,7 +1347,8 @@ sub uri {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetCurrentURL' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetCurrentURL') ]
+    );
     my $response = $self->_get_response($message_id);
     return URI->new( $response->result()->{value} );
 }
@@ -1212,7 +1357,11 @@ sub full_screen {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:FullscreenWindow' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:FullscreenWindow')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1221,7 +1370,8 @@ sub dismiss_alert {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:DismissAlert' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:DismissAlert') ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1243,7 +1393,8 @@ sub accept_dialog {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:AcceptDialog' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:AcceptDialog') ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1252,7 +1403,8 @@ sub alert_text {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetAlertText' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetAlertText') ]
+    );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1286,7 +1438,11 @@ sub selfie {
         }
     }
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:TakeScreenshot', $parameters ] );
+        [
+            _COMMAND(),                                  $message_id,
+            $self->_command('WebDriver:TakeScreenshot'), $parameters
+        ]
+    );
     my $response = $self->_get_response($message_id);
     if ( $extra{hash} ) {
         return $response->result()->{value};
@@ -1315,7 +1471,11 @@ sub current_chrome_window_handle {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetCurrentChromeWindowHandle' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetCurrentChromeWindowHandle')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1324,7 +1484,11 @@ sub chrome_window_handle {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetChromeWindowHandle' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetChromeWindowHandle')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1333,7 +1497,11 @@ sub chrome_window_handles {
     my ( $self, $element ) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetChromeWindowHandles' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetChromeWindowHandles')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return @{ $response->result() };
 }
@@ -1342,7 +1510,11 @@ sub window_handle {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetWindowHandle' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetWindowHandle')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1351,7 +1523,11 @@ sub window_handles {
     my ( $self, $element ) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetWindowHandles' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetWindowHandles')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return @{ $response->result() };
 }
@@ -1360,7 +1536,11 @@ sub close_current_chrome_window_handle {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:CloseChromeWindow' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:CloseChromeWindow')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return @{ $response->result() };
 }
@@ -1369,7 +1549,7 @@ sub close_current_window_handle {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:CloseWindow' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:CloseWindow') ] );
     my $response = $self->_get_response($message_id);
     return @{ $response->result() };
 }
@@ -1379,8 +1559,9 @@ sub css {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'WebDriver:GetElementCSSValue',
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:GetElementCSSValue'),
             { id => $element->uuid(), propertyName => $property_name }
         ]
     );
@@ -1393,8 +1574,9 @@ sub property {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'WebDriver:GetElementProperty',
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:GetElementProperty'),
             { id => $element->uuid(), name => $name }
         ]
     );
@@ -1408,7 +1590,7 @@ sub attribute {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:GetElementAttribute',
+            $self->_command('WebDriver:GetElementAttribute'),
             { id => $element->uuid(), name => $name }
         ]
     );
@@ -1429,8 +1611,10 @@ sub find {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'WebDriver:FindElement', { using => $using, value => $value }
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:FindElement'),
+            { using => $using, value => $value }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1442,7 +1626,10 @@ sub active_frame {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetActiveFrame' ] );
+        [
+            _COMMAND(), $message_id, $self->_command('WebDriver:GetActiveFrame')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     if ( defined $response->result()->{value} ) {
         return Firefox::Marionette::Element->new( $self,
@@ -1456,7 +1643,8 @@ sub active_frame {
 sub title {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
-    $self->_send_request( [ _COMMAND(), $message_id, 'WebDriver:GetTitle' ] );
+    $self->_send_request(
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetTitle') ] );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1480,7 +1668,7 @@ sub quit {
             $self->_send_request(
                 [
                     _COMMAND(), $message_id,
-                    'Marionette:Quit', { flags => $flags }
+                    $self->_command('Marionette:Quit'), { flags => $flags }
                 ]
             );
             my $response = $self->_get_response($message_id);
@@ -1556,7 +1744,7 @@ sub context {
     my ( $self, $new ) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'Marionette:GetContext' ] );
+        [ _COMMAND(), $message_id, $self->_command('Marionette:GetContext') ] );
     my $response = $self->_get_response($message_id);
     my $context  = $response->result()->{value};        # 'content' or 'chrome'
     if ( defined $new ) {
@@ -1564,7 +1752,7 @@ sub context {
         $self->_send_request(
             [
                 _COMMAND(), $message_id,
-                'Marionette:SetContext', { value => $new }
+                $self->_command('Marionette:SetContext'), { value => $new }
             ]
         );
         $response = $self->_get_response($message_id);
@@ -1577,8 +1765,9 @@ sub accept_connections {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'Marionette:AcceptConnections',
+            _COMMAND(),
+            $message_id,
+            $self->_command('Marionette:AcceptConnections'),
             { value => $new ? JSON::true() : JSON::false() }
         ]
     );
@@ -1592,9 +1781,8 @@ sub async_script {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(),
-            $message_id,
-            'WebDriver:ExecuteAsyncScript',
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:ExecuteAsyncScript'),
             { script => $script, %parameters }
         ]
     );
@@ -1607,8 +1795,10 @@ sub script {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'WebDriver:ExecuteScript', { script => $script, %parameters }
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:ExecuteScript'),
+            { script => $script, %parameters }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1620,8 +1810,10 @@ sub html {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id,
-            'WebDriver:GetPageSource', { sessionId => $self->_session_id() }
+            _COMMAND(),
+            $message_id,
+            $self->_command('WebDriver:GetPageSource'),
+            { sessionId => $self->_session_id() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1638,7 +1830,8 @@ sub page_source {
 sub back {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
-    $self->_send_request( [ _COMMAND(), $message_id, 'WebDriver:Back' ] );
+    $self->_send_request(
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:Back') ] );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1646,7 +1839,8 @@ sub back {
 sub forward {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
-    $self->_send_request( [ _COMMAND(), $message_id, 'WebDriver:Forward' ] );
+    $self->_send_request(
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:Forward') ] );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1655,7 +1849,11 @@ sub screen_orientation {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetScreenOrientation' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:GetScreenOrientation')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1664,7 +1862,11 @@ sub switch_to_parent_frame {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:SwitchToParentFrame' ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:SwitchToParentFrame')
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1673,7 +1875,8 @@ sub window_type {
     my ($self) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'WebDriver:GetWindowType' ] );
+        [ _COMMAND(), $message_id, $self->_command('WebDriver:GetWindowType') ]
+    );
     my $response = $self->_get_response($message_id);
     return $response->result()->{value};
 }
@@ -1697,7 +1900,8 @@ sub switch_to_window {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:SwitchToWindow', { name => $window_handle }
+            $self->_command('WebDriver:SwitchToWindow'),
+            { name => $window_handle }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1710,7 +1914,8 @@ sub switch_to_frame {
     $self->_send_request(
         [
             _COMMAND(), $message_id,
-            'WebDriver:SwitchToFrame', { element => $element->uuid() }
+            $self->_command('WebDriver:SwitchToFrame'),
+            { element => $element->uuid() }
         ]
     );
     my $response = $self->_get_response($message_id);
@@ -1722,7 +1927,8 @@ sub go {
     my $message_id = $self->_new_message_id();
     $self->_send_request(
         [
-            _COMMAND(), $message_id, 'WebDriver:Navigate',
+            _COMMAND(), $message_id,
+            $self->_command('WebDriver:Navigate'),
             { url => "$uri", sessionId => $self->_session_id() }
         ]
     );
@@ -1737,7 +1943,7 @@ sub install {
         [
             _COMMAND(),
             $message_id,
-            'Addon:Install',
+            $self->_command('Addon:Install'),
             {
                 path      => "$path",
                 temporary => $temporary ? JSON::true() : JSON::false()
@@ -1752,7 +1958,11 @@ sub uninstall {
     my ( $self, $id ) = @_;
     my $message_id = $self->_new_message_id();
     $self->_send_request(
-        [ _COMMAND(), $message_id, 'Addon:Uninstall', { id => $id } ] );
+        [
+            _COMMAND(), $message_id,
+            $self->_command('Addon:Uninstall'), { id => $id }
+        ]
+    );
     my $response = $self->_get_response($message_id);
     return $self;
 }
@@ -1902,7 +2112,7 @@ Firefox::Marionette - Automate the Firefox browser with the Marionette protocol
 
 =head1 VERSION
 
-Version 0.32
+Version 0.33
 
 =head1 SYNOPSIS
 
@@ -2263,6 +2473,10 @@ returns the pid of the xvfb process if it exists.
 
 Marionette will stop accepting new connections before ending the current session, and finally attempting to quit the application.  This method returns the $? (CHILD_ERROR) value for the Firefox process
 
+=head2 browser_version
+
+This method returns version of firefox.
+
 =head2 child_error
 
 This method returns the $? (CHILD_ERROR) for the Firefox process, or undefined if the process has not yet exited.
